using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.IO;
using AvaCorp.Virtual.BLL;
using AvaCorp.Virtual.Util;
using System.Xml;
using AvaCorp.Virtual.Services.NFe.Cancelamento;
using AvaCorp.Virtual.BLL.CTe.XML;
using AvaCorp.Virtual.BLL.NFe.XML;
using AvaCorp.Virtual.Services.CTe.Cancelamento;

namespace AvaCorp.Virtual.WS
{
    /// <summary>
    /// Summary description for Cancelar
    /// </summary>
    public class Cancelar
    {
        public Cancelar()
        {
        }

        /// <summary>
        /// Verifica se em algum momento o Documento foi cancelado no SEFAZ via sistema AVACORP
        /// </summary>
        /// <param name="pTipoDocumento"></param>
        /// <param name="pGrupo"></param>
        /// <param name="pEmpresa"></param>
        /// <param name="pFilial"></param>
        /// <param name="pUnidade"></param>
        /// <param name="pCnpjCpfCodigoEmissorDocumento"></param>
        /// <param name="pDiferenciadorNumero"></param>
        /// <param name="pSerie"></param>
        /// <param name="pNumero"></param>
        /// <returns></returns>
        private bool VerificaCancelamentoAnterior(int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade, string pCnpjCpfCodigoEmissorDocumento, int pDiferenciadorNumero, int pSerie, int pNumero)
        {
            DocumentoEletronicoCancelamento documentoEletronicoCancelamento = new DocumentoEletronicoCancelamento();
            return documentoEletronicoCancelamento.VerificaCancelamentoAnterior(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero);
        }

        private bool DocumentoEletronicoAutorizado(int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade, string pCnpjCpfCodigoEmissorDocumento, int pDiferenciadorNumero, int pSerie, int pNumero)
        {
            DocumentoEletronicoCancelamento documentoEletronicoCancelamento = new DocumentoEletronicoCancelamento();
            return documentoEletronicoCancelamento.DocumentoEletronicoAutorizado(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero);
        }

        public AvaCorp.Virtual.Types.Retorno DocumentoEletronico(string pChaveAcessoWebService, int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade, string pCnpjCpfCodigoEmissorDocumento, int pDiferenciadorNumero, int pSerie, int pNumero, int pCodigoUsuario, string pJustificativa)
        {
            AvaCorp.Virtual.Types.Retorno retorno = new AvaCorp.Virtual.Types.Retorno();

            #region NT2011/006b - Cancelamento via evento
            // Depois de homologado, o codigo de cancelamento do NFe pode ser removido do cdigo, 
            // mais abaixo, mantendo apenas este codigo
            if (pTipoDocumento == 12)
            {
                try
                {
                    var cancNFe = new CancelamentoNFe();
                    return cancNFe.CancelarPorEvento(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa);
                }
                catch (Exception ex)
                {
                    retorno.CodigoRetorno = false;
                    retorno.DescricaoRetorno = String.Format("Problemas ao Cancelar documento: {0}", ex.Message);
                    return retorno;
                }
            }
            #endregion

            try
            {
                //DALConnections.BeginTransaction();
                //Auditoria.GravaMensagem(String.Format("Cancelar\tChaveAcesso:{0}\tTipoDocumento:{1}\tGrupo:{2}\tEmpresa:{3}\tFilial:{4}\tUnidade:{5}\tCnpjCpfCodigoEmissorDocumento:{6}\tDiferenciadorNumero:{7}\tSerie:{8}\tNumero:{9}\tCodigoUsuario:{10}\tJustificativa:{11}", pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa));
                //Configuracoes.GetParametros(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento);
                int servicoDocumentoEletronico = pTipoDocumento == 12 ? 25 : 7;

                #region Cancelamento por eventos no EDIWS
                #region CTe v2.00/v3.00
                if (pTipoDocumento == 6)
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        //DALConnections.RollbackTransaction();
                        
                        var cancCTe = new CancelamentoCTe();
                        return cancCTe.CancelarPorEvento(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa);
                    }
                    catch (Exception ex)
                    {
                        retorno.CodigoRetorno = false;
                        retorno.DescricaoRetorno = String.Format("Problemas ao Cancelar documento pelo EDIWS: {0}", ex.Message);
                        return retorno;
                    }
                }
                #endregion
                #region NFe v3.10/v4.00
                if (pTipoDocumento == 12)
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        //DALConnections.RollbackTransaction();

                        var cancNFe = new CancelamentoNFe();
                        return cancNFe.CancelarPorEvento(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa);
                    }
                    catch (Exception ex)
                    {
                        retorno.CodigoRetorno = false;
                        retorno.DescricaoRetorno = String.Format("Problemas ao Cancelar documento pelo EDIWS: {0}", ex.Message);
                        return retorno;
                    }
                }
                #endregion
                #endregion

                Types.Estado_ParametroDocumentoEletronico estadoParametroDocumentoEletronico = new Estado_ParametroDocumentoEletronico().Select("BRA", AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, servicoDocumentoEletronico);

                XmlDocument xmlRetornoProcessamento = new XmlDocument();
                DocumentoEletronicoCancelamento documentoEletronicoCancelamento = new DocumentoEletronicoCancelamento();
                Serializacao serializacao = new Serializacao();

                Types.DocumentoEletronicoLote_Documento documento = new BLL.DocumentoEletronicoLote_Documento().Select(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero);

                pJustificativa = pJustificativa.Replace('\r', ' ').Replace('\n', ' ');
                pJustificativa = pJustificativa.Trim().ToUpper().Normalize();
                if ((pJustificativa.Length < 15) || (pJustificativa.Length > 255))
                    throw new Exception("Tamanho da justificativa dever ter entre 15 e 255 caracteres");

                if (!DocumentoEletronicoAutorizado(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero))
                    throw new Exception("Documento no est autorizado no sistema, favor processar o documento eletrnico");
                
                #region Verifica cancelamento anterior
                if (VerificaCancelamentoAnterior(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero))
                {
                    documentoEletronicoCancelamento.CancelaDocumento(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa, documento.DtEmissaoDocumento);
                    DALConnections.CommitTransaction();
                    retorno.CodigoRetorno = true;
                    retorno.DescricaoRetorno = "Documento cancelado com sucesso";
                    Auditoria.GravaMensagem(retorno.DescricaoRetorno);
                    return retorno;
                }
                #endregion

                XmlDocument xmlEnvio = serializacao.CancelaDocumento(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero, pJustificativa, estadoParametroDocumentoEletronico);

                switch (pTipoDocumento)
                {
                    case 6:
                    case 97:
                        CancelamentoCTe cancelamentoCTe = new CancelamentoCTe();
                        TRetCancCTe retCancCTe = cancelamentoCTe.WSCancelaCTe(xmlEnvio, Configuracoes.AmbienteEmissao, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        Auditoria.GravaMensagem(String.Format("Status resposta {0} - Descricao resposta {1}", retCancCTe.infCanc.cStat, retCancCTe.infCanc.xMotivo));
                        xmlRetornoProcessamento = serializacao.GetMemoryXML(typeof(TRetCancCTe), retCancCTe);

                        #region Cancelar documento eletrnico quando j estiver cancelado na SEFAZ
                        if (retCancCTe.infCanc.cStat == "218")
                        {
                            
                            // Daniel Giacomelli - Chamado 33004: Se o documento estiver cancelado no SEFAZ, ento apenas retorna TRUE, que a tela do windev se encarrega de fazer o cancelamento no sistema
                            //documentoEletronicoCancelamento.CancelaDocumento(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero,
                            //    pSerie, pNumero, pCodigoUsuario, pJustificativa, documento.DtEmissaoDocumento);
                            DALConnections.CommitTransaction();
                            retorno.CodigoRetorno = true;
                            retorno.DescricaoRetorno = "Documento cancelado com sucesso";
                            return retorno;
                        }
                        #endregion

                        TRetCancCTeInfCanc retCancCteInfCanc = cancelamentoCTe.RetornaCancelaCTe(retCancCTe);
                        documentoEletronicoCancelamento.InsertByWSCTe(retCancCteInfCanc, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa, xmlEnvio.OuterXml, xmlRetornoProcessamento.OuterXml);
                        DALConnections.CommitTransaction();

                        // Giacomelli 18/02/2013: Apartir de agora, s sero gravados os arquivos de cancelamento homologado
                        if (retCancCTe.infCanc.cStat == "101")
                            GravarXmlRetornoCancelamento(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, retCancCTe.infCanc.dhRecbto, xmlEnvio.InnerXml, xmlRetornoProcessamento.InnerXml, retCancCTe.infCanc.chCTe, retCancCTe.infCanc.nProt);

                        #region retorno para usurio
                        if (retCancCTe.infCanc.cStat == "101" || retCancCTe.infCanc.cStat == "218")
                        {
                            retorno.CodigoRetorno = true;
                            retorno.DescricaoRetorno = "Documento cancelado com sucesso";
                        }
                        else
                        {
                            retorno.CodigoRetorno = false;
                            retorno.DescricaoRetorno = retCancCTe.infCanc.xMotivo;
                        }
                        #endregion

                        break;
                    case 12:
                        CancelamentoNFe cancelamentoNFe = new CancelamentoNFe();
                        TRetCancNFe retCancNFe = cancelamentoNFe.WSCancelaNFe(xmlEnvio, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        Auditoria.GravaMensagem(String.Format("Status resposta {0} - Descricao resposta {1}", retCancNFe.infCanc.cStat, retCancNFe.infCanc.xMotivo));
                        xmlRetornoProcessamento = serializacao.GetMemoryXML(typeof(TRetCancNFe), retCancNFe);

                        #region Cancelar documento eletrnico quando j estiver cancelado na SEFAZ
                        if (retCancNFe.infCanc.cStat == "218")
                        {
                            documentoEletronicoCancelamento.CancelaDocumento(pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero,
                                pSerie, pNumero, pCodigoUsuario, pJustificativa, documento.DtEmissaoDocumento);
                        }
                        #endregion

                        TRetCancNFeInfCanc retCancNFeInfCanc = cancelamentoNFe.RetornaCancelaNFe(retCancNFe);
                        documentoEletronicoCancelamento.InsertByWSNFe(retCancNFeInfCanc, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissorDocumento, pDiferenciadorNumero, pSerie, pNumero, pCodigoUsuario, pJustificativa, xmlEnvio.OuterXml, xmlRetornoProcessamento.OuterXml);
                        DALConnections.CommitTransaction();

                        // Giacomelli 18/02/2013: Apartir de agora, s sero gravados os arquivos de cancelamento homologado
                        if (retCancNFe.infCanc.cStat == "101")
                            GravarXmlRetornoCancelamento(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, retCancNFeInfCanc.dhRecbto, xmlEnvio.InnerXml, xmlRetornoProcessamento.InnerXml, retCancNFeInfCanc.chNFe, retCancNFe.infCanc.nProt);

                        #region retorno para usuario
                        if (retCancNFe.infCanc.cStat == "101" || retCancNFe.infCanc.cStat == "218")
                        {
                            retorno.CodigoRetorno = true;
                            retorno.DescricaoRetorno = "Documento cancelado com sucesso";
                        }
                        else
                        {
                            retorno.CodigoRetorno = false;
                            retorno.DescricaoRetorno = retCancNFe.infCanc.xMotivo;
                        }
                        #endregion

                        break;
                    default:
                        throw new Exception("Documento no especificado");
                }
                return retorno;
            }
            catch (Exception ex)
            {
                DALConnections.RollbackTransaction();
                retorno.CodigoRetorno = false;
                retorno.DescricaoRetorno = String.Format("Problemas ao Cancelar documento: {0}", ex.Message);
                Auditoria.GravaMensagem(String.Format("{0} {1}", retorno.DescricaoRetorno, ex.StackTrace));
                LogSource.Log.GravarMensagem(pTipoDocumento, ex, pCodigoUsuario, "Cancelar", DALConnections.ConnectionString, retorno.DescricaoRetorno);
                return retorno;
            }
        }

        /// <summary>
        /// Grava o XML de cancelamento no banco de dados
        /// </summary>
        private void GravarXmlRetornoCancelamento(int pGrupo, int pEmpresa, string pUF, int pTipoDocumento, DateTime pDtCancelamento, string pXmlEnvio, string pXmlRetornoProcessamento, string pChaveAcesso, string pNumeroProtocolo)
        {
            try
            {
                XmlDocumentoEletronicoCancelamento xmlDocumentoEletronicoCancelamento = new XmlDocumentoEletronicoCancelamento();
                xmlDocumentoEletronicoCancelamento.Insert(pGrupo, pEmpresa, pUF, pTipoDocumento, pDtCancelamento, pXmlEnvio, pXmlRetornoProcessamento, pChaveAcesso, Convert.ToInt64(pNumeroProtocolo));
            }
            catch (Exception ex)
            {
                Auditoria.GravaMensagem(ex.Message);
            }
        }
    }
}