using System;
using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Text.RegularExpressions;
using System.Xml;
using AvaCorp.Virtual.BLL;
using AvaCorp.Virtual.BLL.CTe.XML;
using AvaCorp.Virtual.BLL.NFe.XML;
using AvaCorp.Virtual.Services.CTe.RetornoRecepcao;
using AvaCorp.Virtual.Services.NFe.RetornoRecepcao;
using AvaCorp.Virtual.Util;

namespace AvaCorp.Virtual.WS
{
    /// <summary>
    /// Summary description for Retorno
    /// </summary>
    public class Retorno
    {
        public Retorno() { }

        AvaCorp.Virtual.Types.LogEmailDocumentoEletronico typesLogEmailDocumentoEletronico = new AvaCorp.Virtual.Types.LogEmailDocumentoEletronico();

        public AvaCorp.Virtual.Types.Retorno GetDocumentoEletronicoProcessado(string pChaveAcessoWebService, int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade, string pNumeroRecibo, out string reciboEntregaOriginal)
        {
            AvaCorp.Virtual.Types.Retorno retorno = new AvaCorp.Virtual.Types.Retorno();
            reciboEntregaOriginal = String.Empty;
            try
            {
                #region Envio de Lote no EDIWS
                #region CTe v2.00/v3.00
                if (pTipoDocumento == 6)
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        //DALConnections.RollbackTransaction();

                        var retornoCTe = new Services.CTe.RetornoRecepcao.RetornoRecepcaoCTe();
                        return retornoCTe.RetornoReciboEDIWS(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pNumeroRecibo);
                    }
                    catch (Exception ex)
                    {
                        retorno.CodigoRetorno = false;
                        retorno.DescricaoRetorno = String.Format("Problemas ao consultar o recibo do lote pelo EDIWS: {0}", ex.Message);
                        return retorno;
                    }
                }
                #endregion
                #region NFe v3.10/v4.00
                if (pTipoDocumento == 12)
                {
                    try
                    {
                        // Se vai fazer a transmisso pelo EDIWS, aborta a transao, pois ela no  mais necessria
                        //DALConnections.RollbackTransaction();

                        var retornoNFe = new Services.NFe.RetornoRecepcao.RetornoRecepcaoNFe();
                        return retornoNFe.RetornoReciboEDIWS(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pNumeroRecibo);
                    }
                    catch (Exception ex)
                    {
                        retorno.CodigoRetorno = false;
                        retorno.DescricaoRetorno = String.Format("Problemas ao consultar o recibo do lote pelo EDIWS: {0}", ex.Message);
                        return retorno;
                    }
                }
                #endregion
                #endregion

                AvaCorp.Virtual.BLL.DocumentoEletronicoLote lote = new AvaCorp.Virtual.BLL.DocumentoEletronicoLote();
                AvaCorp.Virtual.BLL.DocumentoEletronicoLote_Documento loteDocumento = new AvaCorp.Virtual.BLL.DocumentoEletronicoLote_Documento();

                #region Begin Transaction / Carrega parmetros do sistema
                DALConnections.BeginTransaction();
                
                // To avoid the concurrency of update for the batch 
                lote.SelectForUpdate(pGrupo, pEmpresa, pTipoDocumento, pNumeroRecibo);

                Auditoria.GravaMensagem(String.Format("Retorno\tChaveAcesso:{0}\tTipoDocumento:{1}\tGrupo:{2}\tEmpresa:{3}\tFilial:{4}\tUnidade:{5}\tNumeroRecibo:{6}", pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pNumeroRecibo));
                Configuracoes.GetParametros(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento);
                AvaCorp.Virtual.BLL.Unidade.Select(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento);
                int servicoDocumentoEletronico = pTipoDocumento == 12 ? 23 : 5;

                BLL.Estado_ParametroDocumentoEletronico estadoBLL = new Estado_ParametroDocumentoEletronico();
                Types.Estado_ParametroDocumentoEletronico estadoParametroDocumentoEletronico = estadoBLL.Select("BRA", AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, servicoDocumentoEletronico);
                List<string> documentoEletronicoRetorno = new BLL.DocumentoEletronicoRetorno().FillCodigos(pTipoDocumento);

                // Carrega o Numero do Lote e Deixa Disponivel
                Int64 numeroLote = lote.SelectNumeroLote(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, Int64.Parse(pNumeroRecibo), Types.Unidade_Parametro.Emitirdocumentoeletronico);

                Auditoria.GravaMensagem(String.Format("Layout version: {0}", estadoParametroDocumentoEletronico.VersaoLayout));

                XmlDocument processaRetorno = new XmlDocument();
                Serializacao serializacao = new Serializacao();

                estadoParametroDocumentoEletronico.TempoEsperaWebservice *= 1000;
                System.Threading.Thread.Sleep(estadoParametroDocumentoEletronico.TempoEsperaWebservice);
                #endregion
                switch (pTipoDocumento)
                {
                    #region CT-e
                    case 6:
                    case 97:

                        #region Busca retorno processamento na SEFAZ
                        RetornoRecepcaoCTe retornoRecepcaoCTe = new RetornoRecepcaoCTe();
                        TRetConsReciCTe retConsReciCTe = retornoRecepcaoCTe.WSRetornoReciboCTe(pNumeroRecibo, pTipoDocumento, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        processaRetorno = serializacao.GetMemoryXML(typeof(TRetConsReciCTe), retConsReciCTe);
                        List<TProtCTeInfProt> protCTeInfProtList = retornoRecepcaoCTe.RetornaDetalhesReciboEnvioCTe(retConsReciCTe);
                        #endregion

                        #region Lote em processamento  - Servio Paralisado (Ocorre bastante no MT e MS)
                        Auditoria.GravaMensagem(String.Format("Status: {0} - {1}", retConsReciCTe.cStat, retConsReciCTe.xMotivo));
                        if ((retConsReciCTe.cStat == "105") || (retConsReciCTe.cStat == "108") || (retConsReciCTe.cStat == "109"))
                        {   //105 - Lote em processamento   108 - Servio Paralisado Momentaneamente (curto prazo)   109 - Servio Paralisado sem Previso
                            retorno.CodigoRetorno = false;
                            retorno.DescricaoRetorno = String.Format("SEFAZ informa: {0}", retConsReciCTe.xMotivo);
                            DALConnections.RollbackTransaction();
                            break;
                        }
                        #endregion

                        #region Atualiza status do lote e documentos no banco / Commit da transao
                        if (!documentoEletronicoRetorno.Contains(retConsReciCTe.cStat)) new BLL.DocumentoEletronicoRetorno().Insert(pTipoDocumento, Convert.ToInt32(retConsReciCTe.cStat), retConsReciCTe.xMotivo);
                        // Ocorrncia: 35738
                        //
                        // PROBLEMA: Foi realizada uma correo neste cdigo. Antes, a situao s era atualizada se existia o protocolo E o cdigo da situao fosse diferente de 106.
                        // Ou seja, se o cdigo fosse diferente de 106 mas no houvesse protocolo no retorno, a situao no era gravada. S era gravada quando as duas
                        // condies eram atendidas. 
                        //
                        // SOLUO: Se a tag do protocolo no constar no XML de retorno de processamento, faz o teste da situao de "Lote no localizado" separadamente.
                        // Neste caso, se no tiver numero do protocolo mas o codigo de resposta for diferente de 106, a situao ser gravada tambm.
                        if (retConsReciCTe.protCTe.Count > 0) // XML com as informaes do CTe
                        {
                            lote.UpdateStatus(pGrupo, pEmpresa, pTipoDocumento, Int64.Parse(pNumeroRecibo), int.Parse(retConsReciCTe.cStat), retConsReciCTe.xMotivo, retConsReciCTe.protCTe[0].infProt.dhRecbto);
                        }
                        else if (retConsReciCTe.cStat != "106") // Lote localizado
                        {
                            lote.UpdateStatus(pGrupo, pEmpresa, pTipoDocumento, Int64.Parse(pNumeroRecibo), int.Parse(retConsReciCTe.cStat), retConsReciCTe.xMotivo, DateTime.Now);
                        }


                        foreach (TProtCTeInfProt cteInfProt in protCTeInfProtList)
                        {
                            if (!documentoEletronicoRetorno.Contains(cteInfProt.cStat)) new BLL.DocumentoEletronicoRetorno().Insert(pTipoDocumento, Convert.ToInt32(cteInfProt.cStat), cteInfProt.xMotivo);
                            Int64 numeroProtocolo = cteInfProt.nProt == null ? 0 : Int64.Parse(cteInfProt.nProt);
                            if ((cteInfProt.dhRecbto == null) || (cteInfProt.dhRecbto.Year < 2000)) cteInfProt.dhRecbto = DateTime.Now;
                            
                            loteDocumento.Update(pGrupo, pEmpresa, pTipoDocumento, cteInfProt.chCTe, numeroProtocolo, int.Parse(cteInfProt.cStat), cteInfProt.dhRecbto, cteInfProt.xMotivo);

                            Auditoria.GravaMensagem(String.Format("{0} - {1} - {2}", cteInfProt.chCTe, cteInfProt.cStat, cteInfProt.xMotivo));
                        }
                        DALConnections.CommitTransaction();
                        #endregion

                        #region Gravar XML de retorno
                        if (retConsReciCTe.protCTe.Count > 0 && retConsReciCTe.cStat != "106") // XML com as informaes do CTe e Lote localizado
                            GravaXmlRetornoProcessamentoDocumentoEletronico(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento, pNumeroRecibo, processaRetorno.InnerXml);

                        #endregion

                        #region Lote no Localizado
                        // A mensagem de lote no localizado pode ocorrer em situaes onde o prazo em que as informaes
                        // do lote ficam disponiveis para consulta do emitente expirou. Atualmente o prazo  15 dias.
                        // Quando isso ocorrer, o sistema deve consulta os documentos um a um, e atualizar a situao no banco.
                        if (retConsReciCTe.cStat == "106")
                        {
                            Auditoria.GravaMensagem("Lote no localizado: O Webservice ir buscar o retorno documento a documento.");
                            this.BuscarRetornorPorDocumento(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, numeroLote);
                        }
                        #endregion

                        #region Procedimentos por documento
                        foreach (TProtCTeInfProt cteInfProt in protCTeInfProtList)
                        {
                            switch (cteInfProt.cStat)
                            {
                                #region Rejeio por duplicidade (Buscar informaes na SEFAZ e enviar um email ao cliente)
                                case "204":
                                    #region Se a sefaz retornou o nmero de recibo ele ser atualizado
                                    reciboEntregaOriginal = System.Text.RegularExpressions.Regex.Match(cteInfProt.xMotivo, @"nRec:(?<recibo>.*)]", System.Text.RegularExpressions.RegexOptions.Singleline | System.Text.RegularExpressions.RegexOptions.IgnoreCase).Groups["recibo"].Value;
                                    if (!String.IsNullOrEmpty(reciboEntregaOriginal))
                                    {
                                        #region Parte do recibo
                                        //Em alguns casos a SEFAZ retorna apenas os ltimos digitos do recibo
                                        if (reciboEntregaOriginal.Length != 15)
                                        {
                                            string reciboInicio = pNumeroRecibo.Substring(0, 3).PadRight(15, '0');
                                            reciboEntregaOriginal = (Convert.ToInt64(reciboInicio) + Convert.ToInt64(reciboEntregaOriginal)).ToString();
                                        }
                                        #endregion
                                        Auditoria.GravaMensagem(String.Format("Encontrado o recibo original: {0}", reciboEntregaOriginal));
                                        AtualizaReciboEntrega(cteInfProt.chCTe, reciboEntregaOriginal);
                                    }
                                    #endregion
                                    
                                    
                                    #region Busca o protocolo de autorizao e atualiza as informaes no sistema
                                    //Giacomelli 2012-10-25: Removido pois sempre deve buscar o numero do protocolo.
                                    //                       Se no tiver o numero do protocolo, no  possvel realizar o cancelamento caso seja necessrio
                                    //else
                                    //{
                                        Types.DocumentoEletronicoLote_Documento doc = loteDocumento.Select(cteInfProt.chCTe);
                                        EnviarEmailDocumentoAutorizadoAnteriormente(doc.TipoDocumento, cteInfProt.chCTe, cteInfProt.cStat, cteInfProt.xMotivo);
                                        AtualizaProtocoloDocumentoEletronicoDuplicado(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, doc.CNPJCPFCodigoEmissorDocumento, doc.DiferenciadorNumeroDocumento, doc.SerieDocumento, doc.NumeroSequenciaDocumento);
                                        using (AvaCorpVirtual ws = new AvaCorpVirtual())
                                        {
                                            ws.ConsultaDocumento(doc.ChaveAcesso);
                                        }
                                    //}
                                    #endregion
                                    break;
                                #endregion

                                #region Inutilizaar documento
                                default:
                                    if ((int.Parse(cteInfProt.cStat) >= 200) && (int.Parse(cteInfProt.cStat) < 999))
                                    {
                                        if (Types.Empresa_Configuracao.InutilizarDocumentoEletronicoAposRejeicao)
                                        {
                                            BLL.DocumentoEletronicoRetorno bllDocumentoEletronicoRetorno = new BLL.DocumentoEletronicoRetorno();
                                            Types.DocumentoEletronicoRetorno documentoEletornicoRetorno = bllDocumentoEletronicoRetorno.SelectTipo(pTipoDocumento, int.Parse(cteInfProt.cStat));
                                            if ((documentoEletornicoRetorno.Tipo == 2) || (documentoEletornicoRetorno.Tipo == 4))
                                                InutilizarDocumentoEletronicoRejeitado(cteInfProt.chCTe, cteInfProt.xMotivo);
                                            else if (documentoEletornicoRetorno.Tipo == 7)
                                            {
                                                throw new Exception(String.Format("{0}, favor tentar processar o documento mais tarde", documentoEletornicoRetorno.Descricao));
                                            }
                                        }
                                    }
                                    break;
                                #endregion
                            }
                        }

                        foreach (Types.DocumentoEletronicoLote_Documento documento in new BLL.DocumentoEletronicoLote_Documento().Select(pTipoDocumento, Convert.ToInt64(pNumeroRecibo)))
                        {
                            InutilizarDocumentoEletronicoRejeitado(documento.ChaveAcesso, "Falha no documento eletrnico");
                        }
                        #endregion

                        retorno.CodigoRetorno = true;
                        retorno.DescricaoRetorno = String.Format("Consulta ao recibo de nmero: {0} efetuada com sucesso!!!", pNumeroRecibo);
                        break;
                    #endregion

                    #region NF-e
                    case 12:
                        #region Busca retorno processamento na SEFAZ
                        RetornoRecepcaoNFe retornoRecepcaoNFe = new RetornoRecepcaoNFe();
                        TRetConsReciNFe retConsReciNFe = retornoRecepcaoNFe.WSRetornoReciboNFe(pNumeroRecibo, estadoParametroDocumentoEletronico, pGrupo, pEmpresa);
                        processaRetorno = serializacao.GetMemoryXML(typeof(TRetConsReciNFe), retConsReciNFe);
                        List<TProtNFeInfProt> protNFeInfProtList = retornoRecepcaoNFe.RetornaDetalhesReciboEnvioNFe(retConsReciNFe);
                        #endregion

                        #region Lote em processamento  - Servio Paralisado (Ocorre bastante no MT e MS)
                        Auditoria.GravaMensagem(String.Format("Status: {0} - {1}", retConsReciNFe.cStat, retConsReciNFe.xMotivo));
                        if ((retConsReciNFe.cStat == "105") || (retConsReciNFe.cStat == "108") || (retConsReciNFe.cStat == "109"))
                        {   //105 - Lote em processamento   108 - Servio Paralisado Momentaneamente (curto prazo)   109 - Servio Paralisado sem Previso
                            retorno.CodigoRetorno = false;
                            retorno.DescricaoRetorno = String.Format("SEFAZ informa: {0}", retConsReciNFe.xMotivo);
                            DALConnections.RollbackTransaction();
                            break;
                        }
                        #endregion

                        #region Atualiza status do lote e documentos no banco / Commit da transao
                        if (!documentoEletronicoRetorno.Contains(retConsReciNFe.cStat)) new BLL.DocumentoEletronicoRetorno().Insert(pTipoDocumento, Convert.ToInt32(retConsReciNFe.cStat), retConsReciNFe.xMotivo);
                        if (retConsReciNFe.protNFe.Count > 0) // XML com as informaes do NFe
                        {
                            lote.UpdateStatus(pGrupo, pEmpresa, pTipoDocumento, Int64.Parse(pNumeroRecibo), int.Parse(retConsReciNFe.cStat), retConsReciNFe.xMotivo, retConsReciNFe.protNFe[0].infProt.dhRecbto);
                        }
                        else if (retConsReciNFe.cStat != "106") // Lote localizado
                        {
                            lote.UpdateStatus(pGrupo, pEmpresa, pTipoDocumento, Int64.Parse(pNumeroRecibo), int.Parse(retConsReciNFe.cStat), retConsReciNFe.xMotivo, DateTime.Now);
                        }


                        foreach (TProtNFeInfProt nfeInfProt in protNFeInfProtList)
                        {
                            if (!documentoEletronicoRetorno.Contains(nfeInfProt.cStat)) new BLL.DocumentoEletronicoRetorno().Insert(pTipoDocumento, Convert.ToInt32(nfeInfProt.cStat), nfeInfProt.xMotivo);
                            Int64 numeroProtocolo = nfeInfProt.nProt == null ? 0 : Int64.Parse(nfeInfProt.nProt);
                            if ((nfeInfProt.dhRecbto == null) || (nfeInfProt.dhRecbto.Year < 2000)) nfeInfProt.dhRecbto = DateTime.Now;
                            loteDocumento.Update(pGrupo, pEmpresa, pTipoDocumento, nfeInfProt.chNFe, numeroProtocolo, int.Parse(nfeInfProt.cStat), nfeInfProt.dhRecbto, nfeInfProt.xMotivo);
                            Auditoria.GravaMensagem(String.Format("{0} - {1} - {2}", nfeInfProt.chNFe, nfeInfProt.cStat, nfeInfProt.xMotivo));
                        }
                        DALConnections.CommitTransaction();
                        #endregion

                        #region Grava XML de retorno
                        if (retConsReciNFe.protNFe.Count > 0 && retConsReciNFe.cStat != "106") // XML com as informaes do CTe e Lote localizado
                        {
                            GravaXmlRetornoProcessamentoDocumentoEletronico(pGrupo, pEmpresa, pFilial, pUnidade, pTipoDocumento, pNumeroRecibo, processaRetorno.InnerXml);
                        }
                        #endregion

                        #region Lote no Localizado
                        // A mensagem de lote no localizado pode ocorrer em situaes onde o prazo em que as informaes
                        // do lote ficam disponiveis para consulta do emitente expirou. Atualmente o prazo  15 dias.
                        // Quando isso ocorrer, o sistema deve consulta os documentos um a um, e atualizar a situao no banco.
                        if (retConsReciNFe.cStat == "106")
                        {
                            Auditoria.GravaMensagem("Lote no localizado: O Webservice ir buscar o retorno documento a documento.");
                            this.BuscarRetornorPorDocumento(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, numeroLote);
                        }
                        #endregion

                        #region Procedimentos por documento
                        foreach (TProtNFeInfProt nfeInfProt in protNFeInfProtList)
                        {
                            switch (nfeInfProt.cStat)
                            {
                                #region Rejeio por duplicidade (Buscar informaes na SEFAZ e enviar um email ao cliente)
                                case "204":
                                    #region Se a sefaz retornou o nmero de recibo ele ser atualizado
                                    reciboEntregaOriginal = Regex.Match(nfeInfProt.xMotivo, @"nRec:(?<recibo>.*)]", RegexOptions.Singleline | RegexOptions.IgnoreCase).Groups["recibo"].Value;
                                    if (!String.IsNullOrEmpty(reciboEntregaOriginal))
                                    {
                                        #region Monta o recibo
                                        //Em alguns casos a SEFAZ retorna apenas os ltimos digitos do recibo
                                        if (reciboEntregaOriginal.Length != 15)
                                        {
                                            string reciboInicio = pNumeroRecibo.Substring(0, 3).PadRight(15, '0');
                                            reciboEntregaOriginal = (Convert.ToInt64(reciboInicio) + Convert.ToInt64(reciboEntregaOriginal)).ToString();
                                        }
                                        #endregion

                                        Auditoria.GravaMensagem(String.Format("Encontrado o recibo original: {0}", reciboEntregaOriginal));
                                        AtualizaReciboEntrega(nfeInfProt.chNFe, reciboEntregaOriginal);
                                    }
                                    #endregion

                                    #region Busca o protocolo de autorizao e atualiza as informaes no sistema
                                    //Giacomelli 2012-10-25: Removido pois sempre deve buscar o numero do protocolo.
                                    //                       Se no tiver o numero do protocolo, no  possvel realizar o cancelamento caso seja necessrio
                                    //else
                                    //{
                                        Types.DocumentoEletronicoLote_Documento doc = loteDocumento.Select(nfeInfProt.chNFe);
                                        EnviarEmailDocumentoAutorizadoAnteriormente(doc.TipoDocumento, nfeInfProt.chNFe, nfeInfProt.cStat, nfeInfProt.xMotivo);
                                        AtualizaProtocoloDocumentoEletronicoDuplicado(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, doc.CNPJCPFCodigoEmissorDocumento, doc.DiferenciadorNumeroDocumento, doc.SerieDocumento, doc.NumeroSequenciaDocumento);
                                        using (AvaCorpVirtual ws = new AvaCorpVirtual())
                                        {
                                            ws.ConsultaDocumento(doc.ChaveAcesso);
                                        }
                                    //}
                                    #endregion
                                    break;
                                #endregion

                                #region Inutilizaar documento
                                default:
                                    if ((int.Parse(nfeInfProt.cStat) >= 200) && (int.Parse(nfeInfProt.cStat) < 999))
                                    {
                                        if (Types.Empresa_Configuracao.InutilizarDocumentoEletronicoAposRejeicao)
                                        {
                                            BLL.DocumentoEletronicoRetorno bllDocumentoEletronicoRetorno = new BLL.DocumentoEletronicoRetorno();
                                            Types.DocumentoEletronicoRetorno documentoEletornicoRetorno = bllDocumentoEletronicoRetorno.SelectTipo(pTipoDocumento, int.Parse(nfeInfProt.cStat));
                                            if ((documentoEletornicoRetorno.Tipo == 2) || (documentoEletornicoRetorno.Tipo == 4))
                                                InutilizarDocumentoEletronicoRejeitado(nfeInfProt.chNFe, nfeInfProt.xMotivo);
                                            else if (documentoEletornicoRetorno.Tipo == 7) {
                                                throw new Exception(String.Format("{0}, favor tentar processar o documento mais tarde",documentoEletornicoRetorno.Descricao));
                                            }
                                        }
                                    }
                                    break;
                                #endregion
                            }
                        }

                        foreach (Types.DocumentoEletronicoLote_Documento documento in new BLL.DocumentoEletronicoLote_Documento().Select(pTipoDocumento, Convert.ToInt64(pNumeroRecibo)))
                        {
                            InutilizarDocumentoEletronicoRejeitado(documento.ChaveAcesso, "Falha no documento eletrnico");
                        }
                        #endregion

                        retorno.CodigoRetorno = true;
                        retorno.DescricaoRetorno = String.Format("Consulta ao recibo de nmero: {0} efetuada com sucesso!!!", pNumeroRecibo);
                        break;
                    #endregion

                    #region Default - Documento no implementado
                    default:
                        Auditoria.GravaMensagem(String.Format("Documento selecionado \"{0}\" no implementado", pTipoDocumento));
                        throw new Exception(String.Format("Documento selecionado \"{0}\" no implementado", pTipoDocumento));
                    #endregion
                }
            }
            catch (Exception ex)
            {
                retorno.CodigoRetorno = false;
                retorno.DescricaoRetorno = String.Format("Problemas ao enviar {0} {1}", ex.Message, ex.StackTrace);
                Auditoria.GravaMensagem(retorno.DescricaoRetorno);
                DALConnections.RollbackTransaction();
                LogSource.Log.GravarMensagem(pTipoDocumento, ex, 1, "Cancelar", DALConnections.ConnectionString, retorno.DescricaoRetorno);
            }
            return retorno;
        }

        private bool EmailConfigurado()
        {
            if ((!String.IsNullOrEmpty(Types.Unidade.EmailEmpresa)) & (!String.IsNullOrEmpty(Types.Unidade.HostSMTP)) & (!String.IsNullOrEmpty(Types.Unidade.EmailName)) & (!String.IsNullOrEmpty(Types.Unidade.EmailPassword)))
                return true;
            else
                return false;
        }

        /// <summary>
        /// Envia um email avisando a transportadora que documento foi autorizado aps uma rejeio de duplicidade, para que o responsvel verifique se est correto
        /// </summary>
        /// <param name="pTipoDocumento"></param>
        /// <param name="pChaveAcessoDocumentoEletronico"></param>
        /// <param name="pCodigoStatus"></param>
        /// <param name="pMotivo"></param>
        private void EnviarEmailDocumentoAutorizadoAnteriormente(int pTipoDocumento, string pChaveAcessoDocumentoEletronico, string pCodigoStatus, string pMotivo)
        {
            try
            {
                if (EmailConfigurado())
                {
                    List<string> emailList = new List<string>();
                    SmtpClient client = new SmtpClient(AvaCorp.Virtual.Types.Unidade.HostSMTP, 25);
                    client.Credentials = new NetworkCredential(AvaCorp.Virtual.Types.Unidade.EmailName, AvaCorp.Virtual.Types.Unidade.EmailPassword);
                    using (MailMessage mensagem = new MailMessage())
                    {
                        AvaCorp.Virtual.Util.Email email = new AvaCorp.Virtual.Util.Email();
                        mensagem.Sender = email.Contato(AvaCorp.Virtual.Types.Empresa_Configuracao.EmailIntegracaoDocumentoEletronico, AvaCorp.Virtual.Types.Empresa_Configuracao.Nome);
                        mensagem.From = email.Contato(AvaCorp.Virtual.Types.Empresa_Configuracao.EmailIntegracaoDocumentoEletronico, AvaCorp.Virtual.Types.Empresa_Configuracao.Nome);
                        emailList.AddRange(Configuracao.EmailDuplicidadeDocumentoEletronico.Split(';'));
                        for (int i = 0; i < emailList.Count; i++)
                            if (emailList[i].Trim().Length > 3)
                                if (IsValidEmail(emailList[i]))
                                {
                                    if (i == 0)
                                        mensagem.To.Add(email.Contato(emailList[i], emailList[i]));
                                    else
                                        mensagem.CC.Add(email.Contato(emailList[i], emailList[i]));
                                }
                                else
                                    Auditoria.GravaMensagem(String.Format("Email invlido\t{0}", emailList[i]));
                        int numeroDocumentoEletronico = int.Parse(pChaveAcessoDocumentoEletronico.Substring(25, 9));
                        mensagem.Subject = String.Format("Rejeio {0} - Filal:{1} - Unidade:{2} - Documento:{3}", pTipoDocumento == 12 ? "NF-e" : "CT-e", Types.Unidade_Parametro.Filial, Types.Unidade_Parametro.Unidade, numeroDocumentoEletronico);
                        StringBuilder sb = new StringBuilder();
                        sb.Append("\tO documento com a chave de acesso nmero {0} retornou com o cdigo de erro: \"{1}-{2}\", mas foi autorizado devido  existncia desta chave na base de dados do SEFAZ.\n\n");
                        sb.Append("\tConsulte a base de dados do SEFAZ para certificar-se que  se trata do mesmo documento.");
                        mensagem.Body = String.Format(sb.ToString(), pChaveAcessoDocumentoEletronico, pCodigoStatus, pMotivo);
                        mensagem.BodyEncoding = Encoding.UTF8;
                        client.Send(mensagem);
                    }
                }
            }
            catch (Exception)
            {
                //nothing to do
            }
        }

        private bool AtualizaReciboEntrega(string chaveAcesso, string reciboEntrega)
        {
            return new DocumentoEletronicoLote(DALConnections.ConnectionString).UpdateReciboEntrega(chaveAcesso, reciboEntrega);
        }

        private void AtualizaProtocoloDocumentoEletronicoDuplicado(string pChaveAcessoWebService, int pTipoDocumento, int pGrupo, int pEmpresa, int pFilial, int pUnidade, string pCnpjCpfCodigoEmissor, int pDiferenciadorNumero, int pSerie, int pNumero)
        {
            SituacaoDocumento consSitDoc = new SituacaoDocumento();
            consSitDoc.AtualizaProtocolo(pChaveAcessoWebService, pTipoDocumento, pGrupo, pEmpresa, pFilial, pUnidade, pCnpjCpfCodigoEmissor, pDiferenciadorNumero, pSerie, pNumero);
        }

        /// <summary>
        /// Grava XML retorno de processamento no banco de dados.
        /// </summary>
        /// <param name="pGrupo"></param>
        /// <param name="pEmpresa"></param>
        /// <param name="pTipoDocumento"></param>
        /// <param name="pNumeroRecibo"></param>
        /// <param name="pRetornoProcessamento"></param>
        private void GravaXmlRetornoProcessamentoDocumentoEletronico(int pGrupo, int pEmpresa, int pFilial, int pUnidade, int pTipoDocumento, string pNumeroRecibo, string pRetornoProcessamento)
        {
            try
            {
                AvaCorp.Virtual.BLL.XmlDocumentoEletronicoLote xmlDocumentoEletronicoLote = new AvaCorp.Virtual.BLL.XmlDocumentoEletronicoLote();
                Int64 numeroLote = xmlDocumentoEletronicoLote.SelectNumeroLote(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, Int64.Parse(pNumeroRecibo), Types.Unidade_Parametro.Emitirdocumentoeletronico);
                if (!xmlDocumentoEletronicoLote.PossuiXML(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, numeroLote, false))
                {
                    Auditoria.GravaMensagem("Lote no possui XML de envio");
                    using (AvaCorpVirtual webservice = new AvaCorpVirtual())
                    {
                        webservice.GerarXmlEnvio(pGrupo, pEmpresa, pTipoDocumento, pFilial, pUnidade, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, 1, numeroLote);
                    }
                }
                //if (!xmlDocumentoEletronicoLote.PossuiXML(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, numeroLote, true))
                xmlDocumentoEletronicoLote.UpdateRetornoProcessamento(pGrupo, pEmpresa, AvaCorp.Virtual.Types.Unidade_Parametro.UFEmissor, pTipoDocumento, numeroLote, pRetornoProcessamento);
            }
            catch (Exception ex)
            {
                Auditoria.GravaMensagem(ex.Message);
            }
        }

        private bool InutilizarDocumentoEletronicoRejeitado(string pChaveAcessoDocumentoEletronico, string pMotivoInutilizacao)
        {
            Types.Retorno retorno = new Types.Retorno();
            Types.DocumentoEletronicoLote_Documento doc = new AvaCorp.Virtual.BLL.DocumentoEletronicoLote_Documento().Select(pChaveAcessoDocumentoEletronico);
            using (AvaCorpVirtual avaCorpVirtual = new AvaCorpVirtual())
            {
                XmlDocument xmlDocumentoInutilizado = new XmlDocument();
                retorno = avaCorpVirtual.InutilizarDocumentoEletronico(Types.Empresa_Configuracao.ChaveAcessoWebServiceDocumentoEletronico, doc.TipoDocumento, doc.Grupo,
                    doc.Empresa, doc.FilialDocumento, doc.UnidadeDocumento, Convert.ToString(doc.SerieDocumento), doc.NumeroSequenciaDocumento, doc.NumeroSequenciaDocumento,
                    DateTime.Today.ToString("yy"), doc.Usuario, pMotivoInutilizacao);
                return retorno.CodigoRetorno;
            }
        }

        /// <summary>
        /// Verifica se a unidade est configurada para enviar os XMLs ao cliente.
        /// </summary>
        /// <param name="pTipoDocumento"></param>
        /// <param name="pChaveAcesso"></param>
        /// <param name="pVersaoLayout"></param>
        private void EmailParametrizado(int pTipoDocumento, string pChaveAcesso, string pVersaoLayout)
        {
            if (Types.Unidade_Parametro.Emitirdocumentoeletronico != 4 && Types.Unidade_Parametro.EnviarXmlAposAutorizacao != 4)
            {
                if (Types.Unidade_Parametro.Emitirdocumentoeletronico == 1 && (Types.Unidade_Parametro.EnviarXmlAposAutorizacao == 1 || Types.Unidade_Parametro.EnviarXmlAposAutorizacao == 3))
                    EmailEnviar(pTipoDocumento, pChaveAcesso, pVersaoLayout);
            }
        }

        private void EmailEnviar(int pTipoDocumento, string pChaveAcesso, string pVersaoLayout)
        {
            Cadastro_Vinculo_Parametrotipodocumento cadastroVinculo = new Cadastro_Vinculo_Parametrotipodocumento();
            AvaCorp.Virtual.Types.Cadastro_Vinculo_ParametroTipoDocumento typesCadastroVinculo = new AvaCorp.Virtual.Types.Cadastro_Vinculo_ParametroTipoDocumento();
            AvaCorp.Virtual.BLL.LogEmailDocumentoEletronico logEmailDocumentoEletronico = new AvaCorp.Virtual.BLL.LogEmailDocumentoEletronico();
            string xmlDistribuicao = String.Empty;
            try
            {
                Auditoria.GravaMensagem(String.Format("Documento: {0}", pChaveAcesso));
                typesCadastroVinculo = cadastroVinculo.Select(pTipoDocumento, pChaveAcesso);
                Auditoria.GravaMensagem(String.Format("Empresa: {0} - E-mail: {1} - Enviar email: {2} - CNPJ pagador frete: {3}", typesCadastroVinculo.RazaoSocial, typesCadastroVinculo.EmailIntegracaoDocumentoEletronico, typesCadastroVinculo.EnviarEmailAposAutorizacao, typesCadastroVinculo.CnpjCpfCodigo));
                if ((typesCadastroVinculo.EnviarEmailAposAutorizacao) && (!String.IsNullOrEmpty(typesCadastroVinculo.EmailIntegracaoDocumentoEletronico)))
                {
                    try
                    {
                        Auditoria.GravaMensagem("Construindo estrutura do email");
                        SmtpClient client = new SmtpClient(AvaCorp.Virtual.Types.Unidade.HostSMTP, AvaCorp.Virtual.Types.Unidade.HostSMTPPort);
                        client.Credentials = new NetworkCredential(AvaCorp.Virtual.Types.Unidade.EmailName, AvaCorp.Virtual.Types.Unidade.EmailPassword);
                        client.EnableSsl = AvaCorp.Virtual.Types.Unidade.EnableSSL;
                        typesLogEmailDocumentoEletronico.Data = DateTime.Now;
                        typesLogEmailDocumentoEletronico.DescricaoDocumento = pChaveAcesso;
                        using (MailMessage mensagemNew = Mensagem(typesCadastroVinculo, pVersaoLayout))
                        {
                            xmlDistribuicao = String.Format(@"{0}{1}_v{2}-proc{3}.xml", Configuracao.CaminhoExportacaoXML, typesCadastroVinculo.NumeroProtocoloDocumentoEletronico, pVersaoLayout, pTipoDocumento == 12 ? "NFe" : "CTe");
                            Attachment anexo = new Attachment(xmlDistribuicao);
                            mensagemNew.Attachments.Add(anexo);
                            client.Send(mensagemNew);
                            anexo.Dispose();
                        }
                        Auditoria.GravaMensagem("E-mail enviado com sucesso!");
                        typesLogEmailDocumentoEletronico.DescricaoStatus = "E-mail enviado com sucesso!";
                        typesLogEmailDocumentoEletronico.Status = 1;
                        logEmailDocumentoEletronico.Insert(typesLogEmailDocumentoEletronico);
                    }
                    catch (Exception ex)
                    {
                        Auditoria.GravaMensagem(ex.Message);
                        typesLogEmailDocumentoEletronico.DescricaoStatus = ex.Message;
                        typesLogEmailDocumentoEletronico.Status = 2;
                        logEmailDocumentoEletronico.Insert(typesLogEmailDocumentoEletronico);
                    }
                    finally
                    {
                        if (File.Exists(xmlDistribuicao))
                            File.Delete(xmlDistribuicao);
                    }
                }
            }
            catch (Exception ex)
            {
                Auditoria.GravaMensagem(String.Format("{0} - {1}", ex.Message, ex.InnerException));
            }
        }

        private MailMessage Mensagem(AvaCorp.Virtual.Types.Cadastro_Vinculo_ParametroTipoDocumento cadastroVinculo, string pVersaoLayout)
        {
            MailMessage mensagem = new MailMessage();
            try
            {
                #region carrega dados log email
                typesLogEmailDocumentoEletronico.Grupo = cadastroVinculo.Grupo;
                typesLogEmailDocumentoEletronico.Empresa = cadastroVinculo.Empresa;
                typesLogEmailDocumentoEletronico.TipoDocumento = cadastroVinculo.TipoDocumento;
                typesLogEmailDocumentoEletronico.Email = cadastroVinculo.EmailIntegracaoDocumentoEletronico;
                #endregion

                AvaCorp.Virtual.Util.Email email = new AvaCorp.Virtual.Util.Email();
                List<string> emailList = new List<string>();
                mensagem.Sender = email.Contato(AvaCorp.Virtual.Types.Empresa_Configuracao.EmailIntegracaoDocumentoEletronico, AvaCorp.Virtual.Types.Empresa_Configuracao.Nome);
                mensagem.From = email.Contato(AvaCorp.Virtual.Types.Empresa_Configuracao.EmailIntegracaoDocumentoEletronico, AvaCorp.Virtual.Types.Empresa_Configuracao.Nome);

                Auditoria.GravaMensagem("Preparando-se para envio de e-mail...");
                
                Auditoria.GravaMensagem(cadastroVinculo.EmailIntegracaoDocumentoEletronico);
                emailList.AddRange(Regex.Match(cadastroVinculo.EmailIntegracaoDocumentoEletronico, @"(?<email>.*?)\.?$", RegexOptions.Singleline | RegexOptions.IgnoreCase).Groups["email"].Value.Split(';'));
                for (int i = 0; i < emailList.Count; i++)
                {
                    if (emailList[i].Trim().Length > 3)
                    {
                        if (IsValidEmail(emailList[i]))
                        {
                            if (i == 0)
                                mensagem.To.Add(email.Contato(emailList[i], cadastroVinculo.RazaoSocial));
                            else
                                mensagem.CC.Add(email.Contato(emailList[i], cadastroVinculo.RazaoSocial));
                        }
                        else
                        {
                            Auditoria.GravaMensagem(String.Format("Email invlido\t{0}", emailList[i]));
                        }
                    }
                }
                mensagem.Subject = String.Format("Xml do {0} {1}", cadastroVinculo.TipoDocumento == 12 ? "NFe" : "CTe", cadastroVinculo.ChaveAcesso);
                mensagem.Body = String.Format("Anexo {0} nmero {1}", cadastroVinculo.TipoDocumento == 12 ? "NFe" : "CTe", cadastroVinculo.ChaveAcesso);
                mensagem.BodyEncoding = Encoding.UTF8;

                using (AvaCorpVirtual avaVirtual = new AvaCorpVirtual())
                {
                    XmlDocument xmlDocument = new XmlDocument();
                    xmlDocument.PreserveWhitespace = true;
                    xmlDocument.LoadXml(avaVirtual.ExportarXmlDynamic(cadastroVinculo.Grupo, cadastroVinculo.Empresa, cadastroVinculo.UFEmissorDocumentoEletronico, cadastroVinculo.TipoDocumento, cadastroVinculo.NumeroLote, cadastroVinculo.ChaveAcesso, pVersaoLayout));
                    xmlDocument.PreserveWhitespace = true;
                    ValidaDocumento validaDocumento = new ValidaDocumento();
                    string xmlPossuiErro = validaDocumento.XML(cadastroVinculo.TipoDocumento, xmlDocument, String.Format("proc{0}_v{1}.xsd", cadastroVinculo.TipoDocumento == 12 ? "NFe" : "CTe", pVersaoLayout));
                    if (xmlPossuiErro != String.Empty)
                        Auditoria.GravaMensagem(xmlPossuiErro);
                    xmlDocument.Save(String.Format(@"{0}{1}_v{2}-proc{3}.xml", Configuracao.CaminhoExportacaoXML, cadastroVinculo.NumeroProtocoloDocumentoEletronico, pVersaoLayout, cadastroVinculo.TipoDocumento == 12 ? "NFe" : "CTe"));
                }
            }
            catch (Exception ex)
            {
                Auditoria.GravaMensagem(ex.Message + ex.InnerException);
            }
            return mensagem;
        }

        /// <summary>
        ///  Retorna true se o formato do email  vlido
        /// </summary>
        /// <param name="email"></param>
        /// <returns></returns>
        private bool IsValidEmail(string email)
        {
            return System.Text.RegularExpressions.Regex.IsMatch(email, @"^([\w-\.]+)@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.)|(([\w-]+\.)+))([a-zA-Z]{2,4}|[0-9]{1,3})(\]?)$");
        }

        /// <summary>
        /// Consulta a situao dos Documento (um a um) e atualizado no banco de dados
        /// </summary>
        /// <param name="pGrupo">Grupo do Lote</param>
        /// <param name="pEmpresa">Empresa do Lote</param>
        /// <param name="pUF">UF do Lote</param>
        /// <param name="pTipoDocumento">Tipo de Documento</param>
        /// <param name="pNumeroLote">Numero do Lote</param>
        private void BuscarRetornorPorDocumento(int pGrupo, int pEmpresa, string pUF, int pTipoDocumento, long pNumeroLote)
        {
            // A mensagem de lote no localizado pode ocorrer em situaes onde o prazo em que as informaes
            // do lote ficam disponiveis para consulta do emitente expirou. Atualmente o prazo  15 dias.
            // Quando isso ocorrer, o sistema deve consulta os documentos um a um, e atualizar a situao no banco.
            BLL.DocumentoEletronicoLote_Documento loteDocumento = new DocumentoEletronicoLote_Documento();
            var documentosLote = loteDocumento.Select(pGrupo, pEmpresa, pUF, pTipoDocumento, pNumeroLote);
            foreach (var doc in documentosLote)
            {
                // Consulta os dados do documento no SEFAZ
                AvaCorp.Virtual.WS.SituacaoDocumento situacaoDocumento = new AvaCorp.Virtual.WS.SituacaoDocumento();
                XmlDocument xmlConsulta = situacaoDocumento.Consultar(doc.ChaveAcesso);

                string cStat = null;
                string xMotivo = null;

                // Obtem o cdigo de status
                Match m = Regex.Match(xmlConsulta.OuterXml, @"<cStat>([0-9]+)</cStat>", RegexOptions.Singleline);
                if (m.Success) cStat = m.Groups[1].Value;

                // Obtem o Motivo
                m = Regex.Match(xmlConsulta.OuterXml, @"<xMotivo>(.*?)</xMotivo>", RegexOptions.Singleline);
                if (m.Success) xMotivo = m.Groups[1].Value;

                // Obtem o Numero do protocolo de autorizao, caso esteja autorizado
                m = Regex.Match(xmlConsulta.OuterXml, @"<nProt>([0-9]+)</nProt>", RegexOptions.Singleline);
                if (m.Success)// Atualiza Situao e Protocolo
                {
                    string cProt = m.Groups[1].Value;
                    Auditoria.GravaMensagem(string.Format("cStat: {0}, xMotivo: {1}, nProt: {2}", cStat, xMotivo, cProt));
                    loteDocumento.Update(doc.ChaveAcesso, int.Parse(cStat), long.Parse(cProt));
                }
                else
                {
                    Auditoria.GravaMensagem(string.Format("cStat: {0}, xMotivo: {1}", cStat, xMotivo));
                    loteDocumento.Update(doc.ChaveAcesso, int.Parse(cStat), 0);
                }
            }
        }

        /// <summary>
        /// Este mtodo Gera o XML de Retorno de Consulta de Recibo de Lote para NFe e CTe e grava na tabela XmlDocumentoEletronicoLote, em casos onde no foi possvel obter o XML do SEFAZ (Ex.: 106 - Lote No Localizado)
        /// </summary>
        /// <param name="pGrupo">Grupo do Lote</param>
        /// <param name="pEmpresa">Empresa do Lote</param>
        /// <param name="pUF">UF do Lote</param>
        /// <param name="pTipoDocumento">Tipo de Documento:
        /// <para>6 - CTe</para>
        /// <para>12 - NFe</para>
        /// </param>
        /// <param name="pNumeroLote">Numero do Lote</param>
        /// <returns>Contedo do XML Gerado</returns>
        public string GeraXmlRetornoProcessamentoPorDocumento(int pGrupo, int pEmpresa, string pUF, int pTipoDocumento, long pNumeroLote)
        {
            try
            {
                AboutJetClass.Select();
                string versaoSistema = String.Format("{0}.{1}.{2}", AvaCorp.Virtual.Types.AboutJetClass.Versao, AvaCorp.Virtual.Types.AboutJetClass.Release, AvaCorp.Virtual.Types.AboutJetClass.SubRelease);

                string tipoDocumentoDescricao = pTipoDocumento == 12 ? "NFe" : "CTe";
                var bllDocumentos = new BLL.DocumentoEletronicoLote_Documento();
                var documentos = bllDocumentos.Select(pGrupo, pEmpresa, pUF, pTipoDocumento, pNumeroLote);

                StringBuilder xmlRetornoProcessamento = new StringBuilder();
                xmlRetornoProcessamento.Append("<?xml version=\"1.0\" encoding=\"utf-8\"?>");
                xmlRetornoProcessamento.AppendFormat("<retConsReci{0} versao=\"{1}\" xmlns=\"http://www.portalfiscal.inf.br/{2}\">", tipoDocumentoDescricao, documentos[0].VersaoLayout, tipoDocumentoDescricao.ToLower());
                xmlRetornoProcessamento.AppendFormat("<tpAmb>{0}</tpAmb>", documentos[0].AmbienteEmissao);
                xmlRetornoProcessamento.AppendFormat("<verAplic>AvaCorp_v{0}</verAplic>", versaoSistema);
                xmlRetornoProcessamento.AppendFormat("<nRec>{0}</nRec>", documentos[0].NumeroReciboEntrega);
                xmlRetornoProcessamento.AppendFormat("<cStat>{0}</cStat>", documentos[0].CodigoStatusRespostaLote);
                xmlRetornoProcessamento.AppendFormat("<xMotivo>{0}</xMotivo>", documentos[0].DescricaoStatusRespostaLote);
                xmlRetornoProcessamento.AppendFormat("<cUF>{0}</cUF>", documentos[0].NumeroReciboEntrega.ToString().Substring(0, 2));
                foreach (Types.DocumentoEletronicoLote_Documento documento in documentos)
                {
                    // Faz um consulta por documento no SEFAZ
                    WS.SituacaoDocumento sitDoc = new SituacaoDocumento();
                    string xmlRetCons = sitDoc.Consultar(documento.ChaveAcesso).OuterXml;

                    // Regex para pegar o elemento PROT do Cte ou Nfe
                    Regex regexProt = new Regex(string.Concat(@"<prot", tipoDocumentoDescricao, @"\b.*?</prot", tipoDocumentoDescricao, @">"), RegexOptions.Singleline);
                    if (regexProt.IsMatch(xmlRetCons))
                    {
                        xmlRetornoProcessamento.Append(regexProt.Match(xmlRetCons).Value);
                    }
                    else
                    {
                        Auditoria.GravaMensagem("Falha na Gerao de XML de Retorno : " + xmlRetCons);
                        return null;
                    }
                }
                xmlRetornoProcessamento.AppendFormat("</retConsReci{0}>", tipoDocumentoDescricao);

                var xml = new XmlDocumentoEletronicoLote();
                xml.UpdateRetornoProcessamento(pGrupo, pEmpresa, pUF, pTipoDocumento, pNumeroLote, xmlRetornoProcessamento.ToString());

                return xmlRetornoProcessamento.ToString();
            }
            catch (Exception e)
            {
                Auditoria.GravaMensagem(e.Message);
                throw new Exception(e.Message);
            }
        }

    }
}